
/* Pocket A/D Converter: Sample Program */

#include <stdio.h>
#include <dos.h>
#include <conio.h>
#include <time.h>
#include <ctype.h>

/* for test only */
#define TEST

/* 12 (or 10) bits resolution */
/* #define RESOLUTION  12 */

#define LPT1 0x3BC
#define LPT2 0x378
#define LPT3 0x278

/* I/O ports for printer interface */
#define DATA(x)         (x + 0)
#define STATUS(x)       (x + 1)
#define CONTROL(x)      (x + 2)

/* definition of each data bit */
#define D7		0x80
#define D6		0x40
#define D5		0x20
#define D4		0x10
#define D3		0x08
#define D2		0x04
#define D1		0x02
#define D0		0x01

/* definition of status port */
#define BUSY	    	0x80
#define ACK         	0x40
#define PAPEROUT    	0x20
#define SELECTED    	0x10
#define ERROR       	0x08

/* definition of control port */
#define IRQENABLE	0x10
#define SELECT      	0x08
#define RESET       	0x04
#define LF          	0x02
#define STROBE      	0x01

/* pin assignments for Pocket A/D converter   */
/* ADDR    : box ID                           */
/* CLK     : clock for serial data in and out */
/* DATAOUT : data in (from host to box)       */
/* DATAIN  : data out (from box to host)      */
/* EOC     : end of coversion                 */
#define	ADDR        	(D7 | D6 | D5 | D4)
#define CLK		D1
#define DATAOUT		D2
#define DATAIN		SELECTED
#define EOC		ERROR

static unsigned char port_value = 0;
static unsigned int port_addr = LPT1;
static unsigned int RESOLUTION = 12;

/* udelay : delay routine */
/* n      : loop count    */
static void
udelay (int n)
{
  int i, j;

  for (i = 0; i < n; i++)
    for (j = 0; j < 10; j++);
}

/* adinit : initial A/D converter       */
/* addr   : I/O address of printer port */
void
adinit(unsigned int addr)
{
  port_addr = addr;
}

/* getad2  : kernel of A/D conversion routine */
/* next_ch : the channel which data           */
/*           will be retrived next            */
/* return  : the A/D value                    */
static int
getad2(int next_ch)
{
  int i, ad;
  unsigned char x;
  clock_t ts;

  /* wait for EOC (end of conversion) */
  /* timeout = 500 ms                 */
  ts = clock();
  do {
    if (inportb (STATUS(port_addr)) & EOC)
      break;
  } while (clock() - ts < 9);
  if (!(inportb (STATUS(port_addr)) & EOC))
    return -1;
  udelay (10);
  /* start retriving data and set next channel */
  ad = 0;
  for (i = 0; i < RESOLUTION; i++) {
    switch (i) {
      case 0:
      case 1:
      case 2:
      case 3:
	x = port_value | ((next_ch & (0x08 >> i))? DATAOUT : 0);
	break;
      default:
	x = port_value;
	break;
    }
    outportb (DATA(port_addr), x);
    udelay (100);
    ad <<= 1;
    if (inportb (STATUS(port_addr)) & DATAIN)
      ad++;
    outportb (DATA(port_addr), x | CLK);
    udelay (20);
    outportb (DATA(port_addr), x);
    udelay (20);
  }
  return ad;
}

/* getad   : get A/D value */
/* box     : box ID        */
/* channel : A/D channel   */
int
getad(int box, int channel)
{
  static int prev_channel = 0;
  static int prev_box = -1;
  int newflag = 0;

  if (box != prev_box) {
    prev_box = box;
    port_value = box << 4;
    outportb (DATA(port_addr), port_value);
    udelay (5);
    newflag = 1;
  }
  if (channel != prev_channel) {
    prev_channel = channel;
    newflag = 1;
  }
  if (newflag)
    getad2 (channel);
  return getad2 (channel);
}

#ifdef TEST

main ()
{
  int lpt = 1;
  int addr = LPT1;
  int box[4] = {0, 1, 2, 3};
  int boxno = 1;
  int bi, channel, i, r;
  unsigned long loopcount = 0;
  int c;
  int runstate = 0;
  int vrange = 10;
  float scale10v = 10.0 / 4096.0;
  float scale5v = 5.0 / 4096.0;
  float voltage;

  clrscr ();
  _setcursortype (_NOCURSOR);
  gotoxy (25, 2);
  cputs ("Pocket A/D Converter Test Program V1.1");
  gotoxy (1, 25);
  cputs ("[L]:LPT [1-4]:BOXID [R]:RESOLUTION [V]:SCALE [G]:GO [S]:STOP [Q]:QUIT [B]:BOXNO ");
  for (channel = 0; channel < 11; channel++) {
    gotoxy (10, 10 + channel);
    cprintf ("%2d", channel + 1);
  }
  adinit (addr);
  do {
    gotoxy (3, 4);
    cprintf ("Loop Count : %-10ldPORT : LPT%d (%3X)   Resolution : %2d   Range : %2d Volt",
	      loopcount, lpt, addr, RESOLUTION, vrange);
    gotoxy (7, 8);
    cputs ("BOX ID");
    gotoxy (7, 9);
    cputs ("Channel");
    for (bi = 0; bi < boxno; bi++) {
      gotoxy (20 + 15 * bi, 7);
      cprintf ("BOX%d  ", bi+1);
      gotoxy (20 + 15 * bi, 8);
      cprintf ("%3d   ", box[bi]);
      gotoxy (16 + 15 * bi, 9);
      cprintf ("Value Voltage", box[bi]);
    }
    if (runstate == 1) {
      loopcount++;
      for (bi = 0; bi < boxno; bi++) {
	for (channel = 0; channel < 11; channel++) {
	  gotoxy (15 + bi * 15, 10 + channel);
	  r = getad(box[bi], channel);
	  voltage = r * ((vrange == 10)? scale10v : scale5v);
	  if (RESOLUTION == 10)
	    voltage = voltage * 4.0;
	  cprintf ("%5d   %5.3f", r, voltage);
	}
      }
    }
    while (kbhit()) {
      c = toupper(getch());
      switch (c) {
	case 'L':
	  if (++lpt > 3)
	    lpt = 1;
	  switch (lpt) {
	    case 1:
	      addr = LPT1;
	      break;
	    case 2:
	      addr = LPT2;
	      break;
	    case 3:
	      addr = LPT3;
	      break;
	  }
	  adinit (addr);
	  break;
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	  i = c - '1';
	  if (++box[i] > 15)
	    box[i] = 0;
	  break;
	case 13:
	case 'G':
	  runstate = 1;
	  break;
	case 'S':
	  runstate = 0;
	  break;
	case 27:
	  if (runstate == 0)
	    runstate = -1;
	  else
	    runstate = 0;
	  break;
	case 'Q':
	  runstate = -1;
	  break;
	case 'B':
	  if (++boxno > 4) {
	    boxno = 1;
	    for (i = 7; i < 24; i++) {
	      gotoxy (19, i);
	      clreol ();
	    }
	  }
	  break;
	case 'R':
	  RESOLUTION = (RESOLUTION == 12)? 10 : 12;
	  break;
	case 'V':
	  vrange = (vrange == 10)? 5 : 10;
	  break;
      } /* switch */
    } /* if */
  } while (runstate != -1);
  clrscr ();
  _setcursortype (_NORMALCURSOR);
  return 0;
}

#endif
