
/* Pocket Digital I/O Converter: Q.C. Program */

#include <stdio.h>
#include <dos.h>
#include <conio.h>
#include <time.h>
#include <ctype.h>

#define LPT1 0x3BC
#define LPT2 0x378
#define LPT3 0x278

/* I/O ports for printer interface */
#define DATA(x)         (x + 0)
#define STATUS(x)       (x + 1)
#define CONTROL(x)      (x + 2)

/* definition of each data bit */
#define D7		0x80
#define D6		0x40
#define D5		0x20
#define D4		0x10
#define D3		0x08
#define D2		0x04
#define D1		0x02
#define D0		0x01

/* definition of status port */
#define BUSY	    	0x80
#define ACK         	0x40
#define PAPEROUT    	0x20
#define SELECTED    	0x10
#define ERROR       	0x08

/* definition of control port */
#define IRQENABLE	0x10
#define SELECT      	0x08
#define RESET       	0x04
#define LF          	0x02
#define STROBE      	0x01

/* pin assignments for Pocket Serious Converter */
/* ADDR    : box ID                             */
/* CLK     : clock for serial data in and out   */
/* DATAIN  : data in (from box to host)         */
/* DATAOUT : data out (from host to box)        */
#define	ADDR        	(D7 | D6 | D5 | D4)
#define DATAOUT		D2
#define CLK		D1
#define DATAIN		SELECTED

/* pin assignments for Pocket A/D converter   */
/* EOC     : end of coversion                 */
#define EOC		ERROR

/* pin assignments for Pocket I/O converter   */
/* LOAD : LOAD data to output                 */
/* RW   : read/write command                  */
#define LOAD		D3
#define RW		D0

static unsigned char port_value = 0;
static unsigned int port_addr = LPT1;
static unsigned int RESOLUTION = 12;
static int prev_box = -1;

/* udelay : delay routine */
/* n      : loop count    */
static void
udelay (int n)
{
  int i, j;

  for (i = 0; i < n; i++)
    for (j = 0; j < 10; j++);
}

/* init : initial converter           */
/* addr : I/O address of printer port */
void
init(unsigned int addr)
{
  port_addr = addr;
  outportb (DATA(port_addr), 0xff);
  udelay (10);
  outportb (DATA(port_addr), 0);
  udelay (10);
}

/* getad2  : kernel of A/D conversion routine */
/* next_ch : the channel which data           */
/*           will be retrived next            */
/* return  : the A/D value                    */
static int
getad2(int next_ch)
{
  int i, ad;
  unsigned char x;
  clock_t ts;

  /* wait for EOC (end of conversion) */
  /* timeout = 500 ms                 */
  ts = clock();
  do {
    if (inportb (STATUS(port_addr)) & EOC)
      break;
  } while (clock() - ts < 9);
  if (!(inportb (STATUS(port_addr)) & EOC))
    return -1;
  udelay (10);
  /* start retriving data and set next channel */
  ad = 0;
  for (i = 0; i < RESOLUTION; i++) {
    switch (i) {
      case 0:
      case 1:
      case 2:
      case 3:
	x = port_value | ((next_ch & (0x08 >> i))? DATAOUT : 0);
	break;
      default:
	x = port_value;
	break;
    }
    outportb (DATA(port_addr), x);
    udelay (100);
    ad <<= 1;
    if (inportb (STATUS(port_addr)) & DATAIN)
      ad++;
    outportb (DATA(port_addr), x | CLK);
    udelay (20);
    outportb (DATA(port_addr), x);
    udelay (20);
  }
  return ad;
}

/* getad   : get A/D value */
/* box     : box ID        */
/* channel : A/D channel   */
int
getad(int box, int channel)
{
  static int prev_channel = 0;

  int newflag = 0;

  if (box != prev_box) {
    prev_box = box;
    port_value = box << 4;
    outportb (DATA(port_addr), port_value);
    udelay (5);
    newflag = 1;
  }
  if (channel != prev_channel) {
    prev_channel = channel;
    newflag = 1;
  }
  if (newflag)
    getad2 (channel);
  return getad2 (channel);
}

/* get8di : get digital input value */
/* box    : box ID       	    */
int
get8di(int box)
{
  int i, r;

  prev_box = -1;
  port_value = (box << 4);
  outportb (DATA(port_addr), port_value);
  udelay (5);
  outportb (DATA(port_addr), port_value | LOAD);
  udelay (5);
  outportb (DATA(port_addr), port_value);
  udelay (5);
  for (r = 0, i = 0; i < 8; i++) {
    if (inportb (STATUS(port_addr)) & DATAIN)
      r |= 0x80 >> i;
    outportb (DATA(port_addr), port_value | CLK);
    udelay (5);
    outportb (DATA(port_addr), port_value);
    udelay (5);
  }
  return r;
}

/* set8do : set digital output value */
/* box    : box ID          	     */
/* val    : output value             */
void
set8do(int box, int val)
{
  int i;

  prev_box = -1;
  port_value = (box << 4) | RW;
  outportb (DATA(port_addr), port_value);
  udelay (5);
  for (i = 0; i < 8; i++) {
    if (val & (0x80 >> i))
      port_value |= DATAOUT;
    else
      port_value &= ~DATAOUT;
    outportb (DATA(port_addr), port_value);
    udelay (5);
    outportb (DATA(port_addr), port_value | CLK);
    udelay (5);
    outportb (DATA(port_addr), port_value);
    udelay (5);
  }
  outportb (DATA(port_addr), port_value | LOAD);
  udelay (5);
  outportb (DATA(port_addr), port_value);
  udelay (5);
}

char *onoff (int flag)
{
  return (flag)? "ON " : "OFF";
}

int
dioloopbacktest(int box)
{
  int i, r, x, y, res;

  for (i = 0, x = 1, y = 0x80; i < 8; i++, x <<= 1, y >>= 1) {
    set8do(box, x);
    delay (100);
    r = get8di (box);
    res = r ^ y;
  }
  return res;
}

#define PKAD	1
#define PKIO	2

main ()
{
  int lpt = 1;
  int addr = LPT1;
  int box[5] = {0, 1, 2, 3, 4};
  int outval[5] = {0, 0, 0, 0, 0};
  int boxtype[5] = {PKAD, PKAD, PKAD, PKAD, PKAD};
  int boxno = 1;
  int currentbox = 0;
  int bi, channel, i, r, mask;
  unsigned long loopcount = 0;
  int c;
  int runstate = 0;
  int vrange = 10;
  float scale10v = 10.0 / 4096.0;
  float scale5v = 5.0 / 4096.0;
  float voltage;

  clrscr ();
  _setcursortype (_NOCURSOR);
  gotoxy (25, 2);
  cputs ("Pocket Converter Test Program V1.3");
  gotoxy (1, 24);
  cputs ("[L]:LPT  [+,-]:BOXID  [A]:A/D  [D]:DIO  [G]:GO  [S]:STOP  [Q]:QUIT  [B]:BOXNO");
  for (channel = 0; channel < 11; channel++) {
    gotoxy (3, 10 + channel);
    cprintf ("%2d", channel + 1);
  }
  init (addr);
  do {
    gotoxy (20, 4);
    cprintf ("Loop Count : %-10ldPort  : LPT%d (%3X)", loopcount, lpt, addr);
    gotoxy (20, 5);
    if (boxtype[currentbox] == PKAD)
      cprintf ("Resolution : %2d        Range : %2d Volt", RESOLUTION, vrange);
    else
      cprintf ("                                          ");
    gotoxy (1, 25);
    if (boxtype[currentbox] == PKAD)
      cputs ("[R] : RESOLUTION  [V] : VOLTAGE          ");
    else
      cputs ("[0-9] : TOGGLE OUTPUT VALUE      ");
    gotoxy (1, 8);
    cputs ("BOX ID");
    gotoxy (1, 9);
    cputs ("Channel");
    for (bi = 0; bi < boxno; bi++) {
      gotoxy (13 + 14 * bi, 7);
      textattr ((bi == currentbox)? 0x70 : 0x07);
      cprintf ("%s #%d  ", (boxtype[bi] == PKAD)? "A/D" : "DIO", bi+1);
      textattr (0x07);
      gotoxy (13 + 14 * bi, 8);
      cprintf ("%3d   ", box[bi]);
      gotoxy (9 + 14 * bi, 9);
      if (boxtype[currentbox] == PKAD)
	cprintf ("Value Voltage");
      else
	cprintf ("Output  Input");
    }
    if (runstate == 1) {
      loopcount++;
      for (bi = 0; bi < boxno; bi++) {
	textattr ((bi == currentbox)? 0x70 : 0x07);
	if (boxtype[bi] == PKAD) {
	  for (channel = 0; channel < 11; channel++) {
	    gotoxy (8 + bi * 14, 10 + channel);
	    r = getad(box[bi], channel);
	    voltage = r * ((vrange == 10)? scale10v : scale5v);
	    if (RESOLUTION == 10)
	      voltage = voltage * 4.0;
	    cprintf ("%5d   %5.3f ", r, voltage);
	  }
	} else {
	  r = get8di(box[bi]);
	  set8do(box[bi], outval[bi]);
	  for (channel = 0, mask = 1; channel < 8; channel++, mask <<= 1) {
	    gotoxy (8 + bi * 14, 10 + channel);
	    cprintf ("  %s    %s  ", onoff(outval[bi] & mask), onoff(r & mask));
	  }
	  textattr (0x07);
	  for (; channel < 11; channel++) {
	    gotoxy (8 + bi * 14, 10 + channel);
	    cputs ("              ");
	  }
	}
      }
      textattr (0x07);
    }
    while (kbhit()) {
      c = toupper(getch());
      if (c == 0 && kbhit()) {
	c = getch ();
	switch (c) {
	  case 75:
	    if (--currentbox < 0)
	      currentbox = boxno - 1;
	    break;
	  case 77:
	    if (++currentbox >= boxno)
	      currentbox = 0;
	    break;
	}
      }
      else {
	switch (c) {
	  case 'L':
	    if (++lpt > 3)
	      lpt = 1;
	    switch (lpt) {
	      case 1:
		addr = LPT1;
		break;
	      case 2:
		addr = LPT2;
		break;
	      case 3:
		addr = LPT3;
		break;
	    }
	    init (addr);
	    break;
	  case '0':
	    outval[currentbox] = 0;
	    break;
	  case '1':
	  case '2':
	  case '3':
	  case '4':
	  case '5':
	  case '6':
	  case '7':
	  case '8':
	    i = 1 << (c - '1');
	    outval[currentbox] ^= i;
	    break;
	  case '9':
	    outval[currentbox] = 0xff;
	    break;
	  case '+':
	    if (++box[currentbox] > 15)
	      box[currentbox] = 0;
	    break;
	  case '-':
	    if (--box[currentbox] < 0)
	      box[currentbox] = 15;
	    break;
	  case 13:
	  case 'G':
	    runstate = 1;
	    break;
	  case 'S':
	    runstate = 0;
	    break;
	  case 27:
	    if (runstate == 0)
	      runstate = -1;
	    else
	      runstate = 0;
	    break;
	  case 'Q':
	    runstate = -1;
	    break;
	  case 'B':
	    if (++boxno > 5) {
	      boxno = 1;
	      currentbox = 0;
	      for (i = 7; i < 24; i++) {
		gotoxy (19, i);
		clreol ();
	      }
	    }
	    break;
	  case 'R':
	    RESOLUTION = (RESOLUTION == 12)? 10 : 12;
	    break;
	  case 'V':
	    vrange = (vrange == 10)? 5 : 10;
	    break;
	  case 'A':
	    boxtype[currentbox] = PKAD;
	    break;
	  case 'D':
	    boxtype[currentbox] = PKIO;
	    break;
	} /* switch */
      } /* else */
    } /* if */
  } while (runstate != -1);
  clrscr ();
  _setcursortype (_NORMALCURSOR);
  return 0;
}
